#ifndef RCL_TGA_HPP
#define RCL_TGA_HPP

#include <fstream>
#include "tga_e.hpp"

class DataBuffer;

namespace tga {

    class TGA {
    public:
	TGA(std::string filename)
	    : file(filename.c_str(), std::ios_base::binary)
	{
	    if (!(file.is_open())){
		throw tga::IOError(filename);
	    }
	}
	~TGA() { file.close(); }
	
	void readHeader() throw(); //throw (tga::TypeError); // rethrows checkType's exception
	unsigned char nextByte() {return this->file.get();}
	bool headerIsValid() const {return this->valid_header;}
	void setHeaderValid(bool valid) {this->valid_header = valid;}
	void checkType()  throw(); //throw (tga::TypeError);

	/* operations on TGA Header fields */
	int getWidth() const {return this->width;}
	void setWidth(int w) {this->width = w;}
	
	int getHeight() const {return this->height;}
	void setHeight(int h) {this->height = h;}
	
	int getType() const {return this->type;}
	void setType(int t) {this->type = t;}
	
	int getBytesPerPixel() const {return this->bytes_per_pixel;}
	void setBytesPerPixel(int bpp) {this->bytes_per_pixel = bpp;}
	
	int getColormapIndex() const {return this->colormap_index;}
	void setColormapIndex(int i) {this->colormap_index = i;}
	
	int getColormapLength() const {return this->colormap_length;}
	void setColormapLength(int l) {this->colormap_length = l;}
	
	int getXOrigin() const {return this->x_origin;}
	void setXOrigin(int x) {this->x_origin = x;} 
	
	int getYOrigin() const {return this->y_origin;}
	void setYOrigin(int y) {this->y_origin = y;}
	
	unsigned char getImageIDSize() const {return this->imageID_size;}
	void setImageIDSize(unsigned char s) {this->imageID_size = s;}
	
	unsigned char getColormapType() const {return this->colormap_type;}
	void setColormapType(unsigned char t) {this->colormap_type = t;}
	
	unsigned char getColormapBits() const {return this->colormap_bits;}
	void setColormapBits(unsigned char bits) {this->colormap_bits = bits;}
	
	unsigned char getImageDescriptor() const {return this->image_descriptor;}
	void setImageDescriptor(unsigned char desc) {this->image_descriptor = desc;}

	int getImageDataOffset() const {return this->imagedata_offset;}
	void setImageDataOffset(int offset) {this->imagedata_offset = offset;}
	
	void seekToData() {this->file.seekg(this->getImageDataOffset(), std::ios_base::beg);}
	void readData(char* data, int size) {this->file.read(data, size);}
	void readData(DataBuffer& data, int size);

    private:
	int imagedata_offset;
	// TGA Header fields 
	int width;
	int height;
	int type;
	int bytes_per_pixel;
	int colormap_index;
	int colormap_length;
	int x_origin;
	int y_origin;	
	unsigned char imageID_size;
	unsigned char colormap_type;
	unsigned char colormap_bits;
	unsigned char image_descriptor;

	bool valid_header;
	std::ifstream file;
    };    
}


#endif 
